/* $Id: dumpprefixes.c,v 1.6 2001/08/12 01:27:23 ghankins Exp $
 *
 * Dumpprefixes - dump BGP prefixes from a file
 *
 * Copyright (C) 2000-2001 Greg Hankins
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
                                                                              */
#include "zebra.h"
#include "prefix.h"
#include "bgpd.h"
#include "bgp_aspath.h"
#include "bgp_route.h"
#include "bgp_dump.h"
#include "bgp_debug.h"
#include "bgp_attr.h"
#include "bgp_community.h"
#include "bgp_clist.h"
#include "bgp_fsm.h"
#include "bgp_packet.h"
#include "bgp_zebra.h"
#include "bgp_open.h"
#include "bgp_filter.h"
#include "vty.h"
#include "memory.h"

#define my_com_nthval(X,n) (*((X).val + (n)))

char *bgp_origin_str[] = {"i","e","?"};

main(int argc, char *argv[])
{
    int fd;
    int count;
    int loopcounter;
    int i;
    int j;
    int k;
    char *l; 
    char c;
    int m;
    int num = -1;
    struct prefix p;
    struct attr attr;
    struct cluster_list cluster;
    struct aspath aspath;
    struct community community;
    int len;
    int summary;
    char buf[BUFSIZ];
    char asbuf[BUFSIZ];
    char combuf[BUFSIZ];
    char prefix[BUFSIZ];
    u_int32_t comval;
    u_int16_t as;
    u_int16_t val;
    extern char *optarg;
    extern int optind, opterr, optopt;

    /* getopt */
    while ((c = getopt(argc, argv, "?hc:n:p:")) != EOF) {
        switch (c) {
            case 'c': {
                summary = 1;
                break;
            }
            case 'n': {
                printf("Not supported yet.\n");
                exit(1);
                num = atoi(optarg);
                break;
            }
            case 'p': {
                printf("Not supported yet.\n");
                exit(1);
                strcpy(prefix, optarg);
                break;
            }
            case 'h':
            default: {
                printf("  %s\n", argv[0]);
                printf("  Usage: %s [-c] [-n <number>] [-p <prefix>] <filename>\n", argv[0]);
                printf("    -c           show prefix count only\n");
                printf("    -n <number>  show this prefix number (default all)\n"
);
                printf("    -p <prefix>  show this prefix (default all)\n");
                exit(1);
                break;
            }
        }
    }

    fd = open(argv[argc - 1], O_RDONLY);
    if (fd < 0) {
        printf("Couldn't open %s.\n", argv[argc - 1]);
        exit(-1);
    }

    /* grab the prefix count from the end */
    lseek(fd, -4, SEEK_END);
    read(fd, &count, sizeof(count));
    printf("File contains %d %s.\n", count, 
        (count == 1) ? "prefix" : "prefixes");

    if (summary == 1) exit(1);

    /* read prefixes */
    lseek(fd, 0, SEEK_SET);
    for (loopcounter = 0; loopcounter < count; loopcounter++) {

        /* struct prefix */
        j = read(fd, &p, sizeof(struct prefix));
        printf("\n  %s/%d\n", inet_ntop(p.family, &p.u.prefix, buf, BUFSIZ), 
            p.prefixlen);

#ifdef HEX
l = (char *)&p;
printf("  ");
for (k = 0; k < j; k++)
printf("%.2X ", *l++);
printf("\n");
#endif

        /* struct attr */
        /* int refcnt;
           u_int32_t flag; */
        j = read(fd, &attr, sizeof(struct attr));

        /* u_char origin; */
        printf("    origin: %s ", bgp_origin_str[attr.origin]);

        /* struct in_addr nexthop; */
        printf("next hop: %s ", inet_ntoa(attr.nexthop));

        /* u_int32_t med; */
        printf("MED: %lu ", attr.med);

        /* u_int32_t local_pref; */
        printf("local pref: %lu ", attr.local_pref);

        /* as_t aggregator_as;
           struct in_addr aggregator_addr;
           u_int32_t dpa;                  */
        /* u_int32_t weight; */
        printf("weight: %lu ", attr.weight);

        /* struct in_addr originator_id; */
        printf("\n    originator ID: %s ", inet_ntoa(attr.originator_id));

#ifdef HEX
l = (char *)&attr;
printf("\n    ");
for (k = 0; k < j; k++)
printf("%.2X ", *l++);
printf("\n");
#endif

        /* struct cluster_list *cluster; - BROKEN */
        read(fd, &j, sizeof(j));
        if (j == -1) {
            printf("\n    no cluster list");
#ifdef HEX
printf("\n    %.2X\n", j);
#endif
        }
        else {
            lseek(fd, -4, SEEK_CUR);
            read(fd, &cluster, sizeof(struct cluster_list));
        }

        /* u_char mp_nexthop_len; */
        /* struct aspath *aspath; */
        read(fd, &j, sizeof(j));
        if (j == -1) {
            printf("\n    no AS path\n");
        }
        else {
            lseek(fd, -4, SEEK_CUR);
            j = read(fd, &aspath, sizeof(struct aspath));
            /* unsigned long refcnt;
               int length;
               int count;            */
            printf("\n    AS path refcnf: %u length: %d count: %d ", aspath.refcnt, aspath.length, aspath.count);

            /* caddr_t data; */
            read(fd, &asbuf, aspath.length);
            /* char *str; */
            printf("\n    AS path str: %x ", aspath.str);
            read(fd, &m, sizeof(m));
            read(fd, &asbuf, m);
            asbuf[m] = '\0';
            printf("AS path: \"%s\"", asbuf);

#ifdef HEX
l = (char *)&aspath;
printf("\n    %d bytes ", j);
for (k = 0; k < j; k++)
printf("%.2X ", *l++);
printf("\n");

printf(" %.2X ", j);

l = (char *)&asbuf;
printf("\n    %d bytes ", j);
for (k = 0; k < j; k++)
printf("%.2X ", *l++);
printf("\n");
#endif

        }

        /* struct community *community; */
        read(fd, &j, sizeof(j));
        if (j == -1) {
            printf("\n    no communities\n");
        }
        else {
            lseek(fd, -4, SEEK_CUR);
            j = read(fd, &i, sizeof(i));
#ifdef HEX
printf("\n    %.2X\n", j);
#endif
            printf("\n    %d communities:", i);
            read(fd, &combuf, i * 4);
            community.refcnt = 0;
            community.size = i;
            community.val = (u_int32_t *)combuf;

            memset(buf, 0, BUFSIZ);
            for (i = 0; i < community.size; i++) {
                comval = ntohl(my_com_nthval(community, i));

                switch (comval) {
                    case COMMUNITY_NO_EXPORT:
                        strlcat(buf, " no-export", BUFSIZ);
                        break;
                    case COMMUNITY_NO_ADVERTISE:
                        strlcat(buf, " no-advertise", BUFSIZ);
                        break;
                    case COMMUNITY_LOCAL_AS:
                        strlcat(buf, " local-AS", BUFSIZ);
                        break;
                    default:
                        as = (comval >> 16) & 0xFFFF;
                        val = comval & 0xFFFF;
                        snprintf(buf + strlen (buf), BUFSIZ - strlen (buf),
                            " %d:%d", as, val);
                        break;
                }
            }
            printf("%s\n", buf);

#ifdef HEX
l = (char *)&community;
printf("\n    ");
for (k = 0; k < sizeof(community); k++)
printf("%.2X%.2X ", *l++, *l++);
printf("\n");
#endif

        }

        /* u_char invalid; */
    }

    printf("\n");
    close(fd);
}
